@@ -5,7 +5,7 @@
  * Routines for DCERPC over SMB packet disassembly
  * Copyright 2001-2003, Tim Potter <tpot@samba.org>
  *
- * $Id: packet-dcerpc-nt.c 23090 2007-10-08 00:32:03Z richardv $
+ * $Id: packet-dcerpc-nt.c 30715 2009-10-26 18:39:51Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -39,6 +39,14 @@
 #include "packet-dcerpc-nt.h"
 #include "packet-windows-common.h"
 
+
+/* This is used to safely walk the decode tree up, one item at a time safely.
+   This is used by dcerpc dissectors that want to push the display of a string
+   higher up in the tree for greater visibility.
+*/
+#define GET_ITEM_PARENT(x) \
+	((x->parent!=NULL)?x->parent:x)
+
 /*
  * This file contains helper routines that are used by the DCERPC over SMB
  * dissectors for wireshark.
@@ -245,15 +253,15 @@ static void cb_byte_array_postprocess(packet_info *pinfo, proto_tree *tree _U_,
 
 	if (levels > 0 && item && s && s[0]) {
 		proto_item_append_text(item, ": %s", s);
-		item = item->parent;
+		item = GET_ITEM_PARENT(item);
 		levels--;
 		if (levels > 0) {
 			proto_item_append_text(item, ": %s", s);
-			item = item->parent;
+			item = GET_ITEM_PARENT(item);
 			levels--;
 			while (levels > 0) {
 				proto_item_append_text(item, " %s", s);
-				item = item->parent;
+				item = GET_ITEM_PARENT(item);
 				levels--;
 			}
 		}
@@ -670,10 +678,10 @@ void dcerpc_store_polhnd_name(e_ctx_hnd *policy_hnd, packet_info *pinfo,
 			if (strcmp(pol->name, name) != 0)
 				g_warning("dcerpc_smb: pol_hash name collision %s/%s\n", value->name, name);
 #endif
-			free(pol->name);
+			/* pol->name is se_allocated, don't free it now */
 		}
 
-		pol->name = strdup(name);
+		pol->name = se_strdup(name);
 
 		return;
 	}
@@ -688,9 +696,9 @@ void dcerpc_store_polhnd_name(e_ctx_hnd *policy_hnd, packet_info *pinfo,
 	pol->last_frame = 0;
 	pol->type = 0;
 	if (name)
-		pol->name = strdup(name);
+		pol->name = se_strdup(name);
 	else
-		pol->name = strdup("<UNKNOWN>");
+		pol->name = se_strdup("<UNKNOWN>");
 
 	add_pol_handle(policy_hnd, pinfo->fd->num, pol, value);
 }
@@ -703,7 +711,7 @@ void dcerpc_store_polhnd_name(e_ctx_hnd *policy_hnd, packet_info *pinfo,
  * close operations?
  */
 
-gboolean dcerpc_fetch_polhnd_data(e_ctx_hnd *policy_hnd, 
+gboolean dcerpc_fetch_polhnd_data(e_ctx_hnd *policy_hnd,
 			      char **name, guint32 *type,
 			      guint32 *open_frame, guint32 *close_frame,
 			      guint32 cur_frame)
@@ -724,7 +732,7 @@ gboolean dcerpc_fetch_polhnd_data(e_ctx_hnd *policy_hnd,
 
 	if (close_frame)
 		*close_frame = 0;
-	
+
 	/* Look up existing value */
 	pol = find_pol_handle(policy_hnd, cur_frame, &value);
 
@@ -745,24 +753,6 @@ gboolean dcerpc_fetch_polhnd_data(e_ctx_hnd *policy_hnd,
 	return pol != NULL;
 }
 
-/* Iterator to free a policy handle key/value pair, and all
-   the policy handle values to which the hash table value
-   points */
-
-static void free_pol_keyvalue(gpointer key _U_, gpointer value_arg,
-    gpointer user_data _U_)
-{
-	pol_hash_value *value = (pol_hash_value *)value_arg;
-	pol_value *pol;
-
-	/* Free user data */
-
-	for (pol = value->list; pol != NULL; pol = pol->next) {
-		free(pol->name);
-		pol->name = NULL;
-	}
-}
-
 /* Initialise policy handle hash */
 
 static void init_pol_hash(void)
@@ -770,7 +760,9 @@ static void init_pol_hash(void)
 	/* Initialise hash table */
 
 	if (pol_hash) {
-		g_hash_table_foreach(pol_hash, free_pol_keyvalue, NULL);
+		/*  Everything in the table is se_ allocated so there's no
+		 *  need to go through and free it all.
+		 */
 		g_hash_table_destroy(pol_hash);
 	}
 
@@ -899,7 +891,7 @@ dissect_nt_hnd(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 	dcerpc_smb_store_pol_pkts(&hnd, pinfo, is_open, is_close);
 
 	/* Insert open/close/name information if known */
-	if (dcerpc_fetch_polhnd_data(&hnd, &name, NULL, &open_frame, 
+	if (dcerpc_fetch_polhnd_data(&hnd, &name, NULL, &open_frame,
 			&close_frame, pinfo->fd->num)) {
 
 		if (open_frame) {
@@ -1148,18 +1140,17 @@ void cb_wstr_postprocess(packet_info *pinfo, proto_tree *tree _U_,
 	}
 
 	/* Append string to upper-level proto_items */
-
 	if (levels > 0 && item && s && s[0]) {
 		proto_item_append_text(item, ": %s", s);
-		item = item->parent;
+		item = GET_ITEM_PARENT(item);
 		levels--;
 		if (levels > 0) {
 			proto_item_append_text(item, ": %s", s);
-			item = item->parent;
+			item = GET_ITEM_PARENT(item);
 			levels--;
 			while (levels > 0) {
 				proto_item_append_text(item, " %s", s);
-				item = item->parent;
+				item = GET_ITEM_PARENT(item);
 				levels--;
 			}
 		}
@@ -1180,7 +1171,7 @@ void cb_str_postprocess(packet_info *pinfo, proto_tree *tree _U_,
 {
 	gint options = GPOINTER_TO_INT(callback_args);
 	gint levels = CB_STR_ITEM_LEVELS(options);
-	char *s;
+	guint8 *s;
 
 	/* Align start_offset on 4-byte boundary. */
 
@@ -1214,15 +1205,15 @@ void cb_str_postprocess(packet_info *pinfo, proto_tree *tree _U_,
 
 	if (levels > 0 && item && s && s[0]) {
 		proto_item_append_text(item, ": %s", s);
-		item = item->parent;
+		item = GET_ITEM_PARENT(item);
 		levels--;
 		if (levels > 0) {
 			proto_item_append_text(item, ": %s", s);
-			item = item->parent;
+			item = GET_ITEM_PARENT(item);
 			levels--;
 			while (levels > 0) {
 				proto_item_append_text(item, " %s", s);
-				item = item->parent;
+				item = GET_ITEM_PARENT(item);
 				levels--;
 			}
 		}
@@ -1327,15 +1318,15 @@ dissect_ndr_nt_SID_with_options(tvbuff_t *tvb, int offset, packet_info *pinfo, p
 
 		if (levels > 0 && item && s && s[0]) {
 			proto_item_append_text(item, ": %s", s);
-			item = item->parent;
+			item = GET_ITEM_PARENT(item);
 			levels--;
 			if (levels > 0) {
 				proto_item_append_text(item, ": %s", s);
-				item = item->parent;
+				item = GET_ITEM_PARENT(item);
 				levels--;
 				while (levels > 0) {
 					proto_item_append_text(item, " %s", s);
-					item = item->parent;
+					item = GET_ITEM_PARENT(item);
 					levels--;
 				}
 			}
@@ -1516,8 +1507,6 @@ dissect_LOGON_HOURS_hours(tvbuff_t *tvb, int offset,
 
 	proto_item_set_len(item, offset-old_offset);
 	return offset;
-
-	return offset;
 }
 
 static gint ett_nt_logon_hours = -1;
