@@ -2,7 +2,7 @@
  *
  * Routines for RFC 2250 MPEG2 (ISO/IEC 13818-1) Transport Stream dissection
  *
- * $Id: packet-mp2t.c 23743 2007-12-04 07:11:38Z jake $
+ * $Id: packet-mp2t.c 26483 2008-10-16 20:21:55Z gerald $
  *
  * Copyright 2006, Erwin Rol <erwin@erwinrol.com>
  *
@@ -187,7 +187,7 @@ static const value_string mp2t_afc_vals[] = {
 };
 
 static gint
-dissect_tsp( tvbuff_t *tvb, gint offset, packet_info *pinfo _U_, proto_tree *tree ) 
+dissect_tsp( tvbuff_t *tvb, gint offset, packet_info *pinfo, proto_tree *tree )
 {
 	guint32 header;
 	guint afc;
@@ -422,10 +422,8 @@ dissect_mp2t( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree )
 {
 	guint offset = 0;
 
-	if (tree) {
-		while ( tvb_reported_length_remaining(tvb, offset) >= MP2T_PACKET_SIZE ) {
-			offset = dissect_tsp( tvb, offset, pinfo, tree);
-		}
+	while ( tvb_reported_length_remaining(tvb, offset) >= MP2T_PACKET_SIZE ) {
+		offset = dissect_tsp( tvb, offset, pinfo, tree);
 	}
 }
 
