@@ -3,7 +3,7 @@
  * <http://www.postgresql.org/docs/current/static/protocol.html>
  * Copyright 2004 Abhijit Menon-Sen <ams@oryx.com>
  *
- * $Id: packet-pgsql.c 23252 2007-10-23 05:50:00Z etxrab $
+ * $Id: packet-pgsql.c 26927 2008-12-05 23:32:25Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -164,10 +164,19 @@ static const value_string format_vals[] = {
 void
 proto_reg_handoff_pgsql(void)
 {
-    dissector_handle_t pgsql_handle;
+    static gboolean initialized = FALSE;
+    static dissector_handle_t pgsql_handle;
+    static guint saved_pgsql_port;
+
+    if (!initialized) {
+        pgsql_handle = create_dissector_handle(dissect_pgsql, proto_pgsql);
+        initialized = TRUE;
+    } else {
+        dissector_delete("tcp.port", saved_pgsql_port, pgsql_handle);
+    }
 
-    pgsql_handle = create_dissector_handle(dissect_pgsql, proto_pgsql);
     dissector_add("tcp.port", pgsql_port, pgsql_handle);
+    saved_pgsql_port = pgsql_port;
 }
 
 
@@ -346,7 +355,7 @@ proto_register_pgsql(void)
     proto_register_field_array(proto_pgsql, hf, array_length(hf));
     proto_register_subtree_array(ett, array_length(ett));
 
-    mod_pgsql = prefs_register_protocol(proto_pgsql, NULL);
+    mod_pgsql = prefs_register_protocol(proto_pgsql, proto_reg_handoff_pgsql);
     prefs_register_uint_preference(
         mod_pgsql, "tcp.port", "PGSQL TCP port", "Set the port for PGSQL "
         "messages (if different from the default of 5432)", 10, &pgsql_port
