@@ -8,7 +8,7 @@
  * Dissection of multiple SMPP PDUs within one packet
  * provided by Chris Wilson.
  *
- * $Id: packet-smpp.c 22908 2007-09-19 17:33:08Z martinm $
+ * $Id: packet-smpp.c 27265 2009-01-18 21:07:06Z gerald $
  *
  * Refer to the AUTHORS file or the AUTHORS section in the man page
  * for contacting the author(s) of this file.
@@ -49,10 +49,13 @@
 #include <glib.h>
 
 #include <epan/packet.h>
+#include <epan/tap.h>
+#include <epan/stats_tree.h>
 
 #include <epan/prefs.h>
 #include <epan/emem.h>
 #include "packet-tcp.h"
+#include "packet-smpp.h"
 
 /* General-purpose debug logger.
  * Requires double parentheses because of variable arguments of printf().
@@ -68,6 +71,8 @@
 #define DebugLog(x) ;
 #endif
 
+#define SMPP_MIN_LENGTH 16
+
 /* Forward declarations		*/
 static void dissect_smpp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree);
 static guint get_smpp_pdu_len(packet_info *pinfo, tvbuff_t *tvb, int offset);
@@ -80,6 +85,11 @@ static void dissect_smpp_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree
  */
 static int proto_smpp				= -1;
 
+static int st_smpp_ops				= -1;
+static int st_smpp_req				= -1;
+static int st_smpp_res				= -1;
+static int st_smpp_res_status			= -1;
+
 static int hf_smpp_command_id			= -1;
 static int hf_smpp_command_length		= -1;
 static int hf_smpp_command_status		= -1;
@@ -213,6 +223,12 @@ static gint ett_dcs		= -1;
 /* Reassemble SMPP TCP segments */
 static gboolean reassemble_over_tcp = TRUE;
 
+/* Tap */
+static int smpp_tap		= -1;
+
+/* Stats Tree */
+static guint8* st_str_smpp = "SMPP Operations";
+
 /*
  * Value-arrays for field-contents
  */
@@ -728,6 +744,47 @@ static const value_string vals_dcs_wap_charset[] = {
 
 static dissector_handle_t gsm_sms_handle;
 
+/*
+ * For Stats Tree
+ */
+void
+smpp_stats_tree_init(stats_tree* st)
+{
+	st_smpp_ops = stats_tree_create_node(st, "SMPP Operations", 0, TRUE);
+	st_smpp_req = stats_tree_create_node(st, "SMPP Requests", st_smpp_ops, TRUE);
+	st_smpp_res = stats_tree_create_node(st, "SMPP Responses", st_smpp_ops, TRUE);
+	st_smpp_res_status = stats_tree_create_node(st, "SMPP Response Status", 0, TRUE);
+
+}
+
+int
+smpp_stats_tree_per_packet(stats_tree *st, /* st as it was passed to us */
+                                      packet_info *pinfo _U_,
+                                      epan_dissect_t *edt _U_,
+                                      const void *p) /* Used for getting SMPP command_id values */
+{
+	smpp_tap_rec_t* tap_rec = (smpp_tap_rec_t*)p;
+
+	tick_stat_node(st, "SMPP Operations", 0, TRUE);
+
+	if ((tap_rec->command_id & 0x80000000) == 0x80000000) /* Response */
+	{
+		tick_stat_node(st, "SMPP Responses", st_smpp_ops, TRUE);
+		tick_stat_node(st, val_to_str(tap_rec->command_id, vals_command_id, "Unknown 0x%08x"), st_smpp_res, FALSE);
+
+		tick_stat_node(st, "SMPP Response Status", 0, TRUE);
+		tick_stat_node(st, val_to_str(tap_rec->command_status, vals_command_status, "Unknown 0x%08x"), st_smpp_res_status, FALSE);
+
+	}
+	else  /* Request */
+	{
+		tick_stat_node(st, "SMPP Requests", st_smpp_ops, TRUE);
+		tick_stat_node(st, val_to_str(tap_rec->command_id, vals_command_id, "Unknown 0x%08x"), st_smpp_req, FALSE);
+	}
+
+	return 1;
+}
+
 /*!
  * SMPP equivalent of mktime() (3). Convert date to standard 'time_t' format
  *
@@ -1050,12 +1107,18 @@ smpp_handle_tlv(proto_tree *tree, tvbuff_t *tvb, int *offset)
 				 hf_smpp_privacy_indicator, offset);
 		break;
 	    case  0x0202:	/* source_subaddress	*/
-		smpp_handle_string(sub_tree, tvb,
-				   hf_smpp_source_subaddress, offset);
+		if (length) {
+		    proto_tree_add_item(sub_tree, hf_smpp_source_subaddress, 
+				    tvb, *offset, length, FALSE);
+                    (*offset) += length;
+		}
 		break;
 	    case  0x0203:	/* dest_subaddress	*/
-		smpp_handle_string(sub_tree, tvb,
-				   hf_smpp_dest_subaddress, offset);
+		if (length) {
+		    proto_tree_add_item(sub_tree, hf_smpp_dest_subaddress, 
+				    tvb, *offset, length, FALSE);
+                    (*offset) += length;
+		}
 		break;
 	    case  0x0204:	/* user_message_reference	*/
 		smpp_handle_int2(sub_tree, tvb,
@@ -1140,7 +1203,6 @@ smpp_handle_tlv(proto_tree *tree, tvbuff_t *tvb, int *offset)
 				 hf_smpp_network_error_type, offset);
 		smpp_handle_int2(sub_tree, tvb,
 				 hf_smpp_network_error_code, offset);
-		(*offset) += length;
 		break;
 	    case  0x0424:	/* message_payload	*/
 		if (length)
@@ -1181,7 +1243,6 @@ smpp_handle_tlv(proto_tree *tree, tvbuff_t *tvb, int *offset)
 		proto_tree_add_item(sub_tree,
 			    	    hf_smpp_alert_on_message_delivery,
 				    tvb, *offset, length, FALSE);
-		(*offset) += length;
 		break;
 	    case  0x1380:	/* its_reply_type	*/
 		smpp_handle_int1(sub_tree, tvb,
@@ -1649,10 +1710,10 @@ dissect_smpp_heur(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     guint	 command_status;	/* Status code		*/
     guint	 command_length;	/* length of PDU	*/
 
-    if (tvb_reported_length(tvb) < 4 * 4)	/* Mandatory header	*/
+    if (tvb_reported_length(tvb) < SMPP_MIN_LENGTH)	/* Mandatory header	*/
 	return FALSE;
     command_length = tvb_get_ntohl(tvb, 0);
-    if (command_length > 64 * 1024)
+    if (command_length > 64 * 1024 || command_length < SMPP_MIN_LENGTH)
 	return FALSE;
     command_id = tvb_get_ntohl(tvb, 4);		/* Only known commands	*/
     if (match_strval(command_id, vals_command_id) == NULL)
@@ -1722,6 +1783,7 @@ dissect_smpp_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     guint	 command_id;		/* SMPP command		*/
     guint	 command_status;	/* Status code		*/
     guint	 sequence_number;	/* ...of command	*/
+    smpp_tap_rec_t* tap_rec;		/* Tap record		*/
     const gchar	*command_str;
     const gchar	*command_status_str = NULL;
     /* Set up structures needed to add the protocol subtree and manage it */
@@ -1732,7 +1794,7 @@ dissect_smpp_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      * Safety: don't even try to dissect the PDU
      * when the mandatory header isn't present.
      */
-    if (tvb_reported_length(tvb) < 4 * 4)
+    if (tvb_reported_length(tvb) < SMPP_MIN_LENGTH)
 	return;
     command_length = tvb_get_ntohl(tvb, offset);
     offset += 4;
@@ -1955,9 +2017,17 @@ dissect_smpp_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 			    break;
 		    } /* switch (command_id) */
 		} /* if (command_id & 0x80000000) */
+
 	    } /* if (command_length <= tvb_reported_length(pdu_tvb)) */
 	    offset += command_length;
 	} /* if (tree || (command_id == 4)) */
+
+	/* Queue packet for Tap */
+	tap_rec = ep_alloc0(sizeof(smpp_tap_rec_t));
+	tap_rec->command_id = command_id;
+	tap_rec->command_status = command_status;
+	tap_queue_packet(smpp_tap, pinfo, tap_rec);
+
 	first = FALSE;
     }
 
@@ -2396,14 +2466,14 @@ proto_register_smpp(void)
 	},
     {   &hf_smpp_source_subaddress,
 	    {   "Source Subaddress", "smpp.source_subaddress",
-		FT_STRING, BASE_NONE, NULL, 0x00,
+		FT_BYTES, BASE_HEX, NULL, 0x00,
 		"Source Subaddress",
 		HFILL
 	    }
 	},
     {   &hf_smpp_dest_subaddress,
 	    {   "Destination Subaddress", "smpp.dest_subaddress",
-		FT_STRING, BASE_NONE, NULL, 0x00,
+		FT_BYTES, BASE_HEX, NULL, 0x00,
 		"Destination Subaddress",
 		HFILL
 	    }
@@ -2757,6 +2827,9 @@ proto_register_smpp(void)
     /* Allow other dissectors to find this one by name. */
     register_dissector("smpp", dissect_smpp, proto_smpp);
 
+    /* Register for tapping */
+    smpp_tap = register_tap("smpp");
+
     /* Preferences */
     smpp_module = prefs_register_protocol (proto_smpp, NULL);
     prefs_register_bool_preference (smpp_module,
@@ -2791,8 +2864,11 @@ proto_reg_handoff_smpp(void)
     heur_dissector_add("tcp", dissect_smpp_heur, proto_smpp);
     heur_dissector_add("x.25", dissect_smpp_heur, proto_smpp);
 
-	/* Required for call_dissector() */
-	DebugLog(("Finding gsm-sms-ud subdissector\n"));
-	gsm_sms_handle = find_dissector("gsm-sms-ud");
-	DISSECTOR_ASSERT(gsm_sms_handle);
+    /* Required for call_dissector() */
+    DebugLog(("Finding gsm-sms-ud subdissector\n"));
+    gsm_sms_handle = find_dissector("gsm-sms-ud");
+    DISSECTOR_ASSERT(gsm_sms_handle);
+
+    /* Tapping setup */
+    stats_tree_register("smpp","smpp_commands", st_str_smpp, smpp_stats_tree_per_packet, smpp_stats_tree_init, NULL);
 }
