@@ -2,7 +2,7 @@
  * Definitions for Wireshark memory management and garbage collection
  * Ronnie Sahlberg 2005
  *
- * $Id: emem.h 19877 2006-11-11 22:39:25Z sahlberg $
+ * $Id: emem.h 23736 2007-12-04 03:26:50Z sahlberg $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -69,6 +69,11 @@ gchar* ep_strdup_printf(const gchar* fmt, ...)
 /* allocates with a packet lifetime scope an array of type made of num elements */
 #define ep_alloc_array(type,num) (type*)ep_alloc(sizeof(type)*(num))
 
+/* allocates with a packet lifetime scope an array of type made of num elements,
+ * initialised to zero.
+ */
+#define ep_alloc_array0(type,num) (type*)ep_alloc0(sizeof(type)*(num))
+
 /* 
  * Splits a string into a maximum of max_tokens pieces, using the given
  * delimiter. If max_tokens is reached, the remainder of string is appended
@@ -160,14 +165,17 @@ void se_free_all(void);
 /**************************************************************
  * binary trees 
  **************************************************************/
-#define EMEM_TREE_RB_COLOR_RED		0x00
-#define EMEM_TREE_RB_COLOR_BLACK	0x01
 typedef struct _emem_tree_node_t {
 	struct _emem_tree_node_t *parent;
 	struct _emem_tree_node_t *left;
 	struct _emem_tree_node_t *right;
-	union {
-		guint32 rb_color;
+	struct {
+#define EMEM_TREE_RB_COLOR_RED		0
+#define EMEM_TREE_RB_COLOR_BLACK	1
+		guint32 rb_color:1;
+#define EMEM_TREE_NODE_IS_DATA		0
+#define EMEM_TREE_NODE_IS_SUBTREE	1
+		guint32 is_subtree:1;
 	} u;
 	guint32 key32;
 	void *data;
@@ -183,7 +191,7 @@ typedef struct _emem_tree_node_t {
 typedef struct _emem_tree_t {
 	struct _emem_tree_t *next;
 	int type;
-	char *name;    /* just a string to make debugging easier */
+	const char *name;    /* just a string to make debugging easier */
 	emem_tree_node_t *tree;
 	void *(*malloc)(size_t);
 } emem_tree_t;
@@ -203,7 +211,7 @@ extern emem_tree_t *se_trees;
  *
  * type is : EMEM_TREE_TYPE_RED_BLACK for a standard red/black tree.
  */
-emem_tree_t *se_tree_create(int type, char *name);
+emem_tree_t *se_tree_create(int type, const char *name);
 
 /* This function is similar to the se_tree_create() call but with the
  * difference that when the se memory is release everything including the 
@@ -214,7 +222,7 @@ emem_tree_t *se_tree_create(int type, char *name);
  * another structure that is also se allocated so that when the structure is
  * released, the tree will be completely released as well.
  */
-emem_tree_t *se_tree_create_non_persistent(int type, char *name);
+emem_tree_t *se_tree_create_non_persistent(int type, const char *name);
 
 /* se_tree_insert32 
  * Insert data into the tree and key it by a 32bit integer value
@@ -253,6 +261,9 @@ emem_tree_t *se_tree_create_non_persistent(int type, char *name);
 /* Lookup the value under a string key */
 #define se_tree_lookup_string emem_tree_lookup_string
 
+/* Traverse a tree */
+#define se_tree_foreach emem_tree_foreach
+
 
 /* *******************************************************************
  * Tree functions for PE memory allocation scope
@@ -267,6 +278,7 @@ emem_tree_t *pe_tree_create(int type, char *name);
 #define pe_tree_lookup32_array emem_tree_lookup32_array
 #define pe_tree_insert_string emem_tree_insert_string
 #define pe_tree_lookup_string emem_tree_lookup_string
+#define pe_tree_foreach emem_tree_foreach
 
 
 
@@ -304,6 +316,10 @@ typedef struct _emem_tree_key_t {
  * The data pointer should be allocated by SE allocators so that the
  * data will be released at the same time as the tree itself is destroyed.
  *
+ * Note: all the "key" members of the "key" argument MUST be aligned on
+ * 32-bit boundaries; otherwise, this code will crash on platforms such
+ * as SPARC that require aligned pointers.
+ *
  * If you use ...32_array() calls you MUST make sure that every single node
  * you add to a specific tree always has a key of exactly the same number of 
  * keylen words or things will most likely crash. Or at least that every single
@@ -333,13 +349,20 @@ void emem_tree_insert32_array(emem_tree_t *se_tree, emem_tree_key_t *key, void *
  */
 void *emem_tree_lookup32_array(emem_tree_t *se_tree, emem_tree_key_t *key);
 
+/* case insensitive strings as keys */
+#define EMEM_TREE_STRING_NOCASE			0x00000001
 /* Insert a new value under a string key */
-void emem_tree_insert_string(emem_tree_t* h, const gchar* k, void* v);
+void emem_tree_insert_string(emem_tree_t* h, const gchar* k, void* v, guint32 flags);
 
 /* Lookup the value under a string key */
-void* emem_tree_lookup_string(emem_tree_t* h, const gchar* k);
+void* emem_tree_lookup_string(emem_tree_t* h, const gchar* k, guint32 flags);
 
 
+/* traverse a tree. if the callback returns TRUE the traversal will end */
+typedef gboolean (*tree_foreach_func)(void *value, void *userdata);
+
+gboolean emem_tree_foreach(emem_tree_t* emem_tree, tree_foreach_func callback, void *user_data);
+
 
 
 
