@@ -2,7 +2,7 @@
  * API for a counter tree for Wireshark
  * 2004, Luis E. G. Ontanon
  *
- * $Id: stats_tree.c 20620 2007-01-30 02:11:47Z gerald $
+ * $Id: stats_tree.c 21253 2007-03-28 21:55:11Z sfisher $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -29,6 +29,7 @@
 
 #include <glib.h>
 #include <epan/stats_tree_priv.h>
+#include <epan/ws_strsplit.h>
 #include <string.h>
 
 #include "stats_tree.h"
@@ -389,11 +390,11 @@ extern void stats_tree_presentation(void (*registry_iterator)(gpointer,gpointer,
 *    as_named_node: whether or not it has to be registered in the root namespace
 */
 static stat_node*  new_stat_node(stats_tree* st,
-								 const gchar* name,
-								 int parent_id,
-								 gboolean with_hash,
-								 gboolean as_parent_node) {
-	
+				 const gchar* name,
+				 int parent_id,
+				 gboolean with_hash,
+				 gboolean as_parent_node) {
+
 	stat_node *node = g_malloc (sizeof(stat_node));
 	stat_node* last_chld = NULL;
 	
@@ -541,48 +542,42 @@ extern guint8* stats_tree_get_abbr(const guint8* optarg) {
  *
  */
 static range_pair_t* get_range(guint8* rngstr) {
-	gchar** split;
-	range_pair_t* rng;
-	
-	split = g_strsplit((gchar*)rngstr,"-",2);
+        gchar** split;
+        range_pair_t* rng;
 
-	/* empty string */
-	if (split[0] == NULL) {
-	  g_strfreev(split);
-	  return NULL;
-	}
+        split = g_strsplit((gchar*)rngstr,"-",2);
 
-#if GLIB_MAJOR_VERSION >= 2
-	/* means we have a non empty string 
-	 * which does not contain a delimiter */
-	if (split[1] == NULL) {
-	  g_strfreev(split);
-	  return NULL;
-	}
-#endif
+        /* empty string */
+        if (split[0] == NULL) {
+          g_strfreev(split);
+          return NULL;
+        }
 
-	rng = g_malloc(sizeof(range_pair_t));
+        /* means we have a non empty string 
+         * which does not contain a delimiter */
+        if (split[1] == NULL) {
+          g_strfreev(split);
+          return NULL;
+        }
 
-	/* string == "X-?" */
-	if (*(split[0]) != '\0') {
-	    rng->floor = strtol(split[0],NULL,10);
-	} else
-	  /* string == "-?" */
-	  rng->floor = G_MININT;
+        rng = g_malloc(sizeof(range_pair_t));
+
+        /* string == "X-?" */
+        if (*(split[0]) != '\0') {
+            rng->floor = strtol(split[0],NULL,10);
+        } else
+          /* string == "-?" */
+          rng->floor = G_MININT;
+
+        /* string != "?-" */
+        if (*(split[1]) != '\0') {
+          rng->ceil  = strtol(split[1],NULL,10);
+        } else
+          /* string == "?-" */
+          rng->ceil = G_MAXINT;
+
+        g_strfreev(split);
 
-	/* string != "?-" */
-#if GLIB_MAJOR_VERSION >= 2
-	if (*(split[1]) != '\0') {
-#else
-	if (split[1] != NULL) {
-#endif
-	  rng->ceil  = strtol(split[1],NULL,10);
-	} else
-	  /* string == "?-" */
-	  rng->ceil = G_MAXINT;
-	
-	g_strfreev(split);
-	
 	return rng;
 }
 
@@ -595,7 +590,7 @@ extern int stats_tree_create_range_node(stats_tree* st,
 	guint8* curr_range;
 	stat_node* rng_root = new_stat_node(st, name, parent_id, FALSE, TRUE);
 	stat_node* range_node = NULL;
-	
+
 	va_start( list, parent_id );
 	while (( curr_range = va_arg(list, guint8*) )) {
 		range_node = new_stat_node(st, curr_range, rng_root->id, FALSE, FALSE);
@@ -702,8 +697,8 @@ extern int stats_tree_create_pivot_by_pname(stats_tree* st,
 }
 
 extern int stats_tree_tick_pivot(stats_tree* st,
-					  int pivot_id,
-					  const gchar* pivot_value) {
+				 int pivot_id,
+				 const gchar* pivot_value) {
 	
 	stat_node* parent = g_ptr_array_index(st->parents,pivot_id);
 	
