@@ -4,7 +4,7 @@
  * Helper routines common to all composite expert statistics
  * tap.
  *
- * $Id: expert_comp_table.c 23715 2007-12-03 21:41:06Z gerald $
+ * $Id: expert_comp_table.c 23814 2007-12-09 09:02:14Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -76,25 +76,25 @@ gpointer userdata)
     {
         case SORT_ALPHABETICAL:
         {
-        gchar *name1, *name2;
-        gtk_tree_model_get(model, a, 0, &name1, -1);
-        gtk_tree_model_get(model, b, 0, &name2, -1);
-        if (name1 == NULL || name2 == NULL)
-        {
-            if (name1 == NULL && name2 == NULL)
-                break; /* both equal => ret = 0 */
-            ret = (name1 == NULL) ? -1 : 1;
-        }
-        else
-        {
-            ret = g_ascii_strcasecmp(name1,name2);
-        }
-        g_free(name1);
-        g_free(name2);
-        }
-        break;
+            gchar *name1, *name2;
+            gtk_tree_model_get(model, a, 0, &name1, -1);
+            gtk_tree_model_get(model, b, 0, &name2, -1);
+            if (name1 == NULL || name2 == NULL)
+            {
+                if (name1 == NULL && name2 == NULL)
+                    break; /* both equal => ret = 0 */
+                ret = (name1 == NULL) ? -1 : 1;
+            }
+            else
+            {
+                ret = g_ascii_strcasecmp(name1,name2);
+            }
+            g_free(name1);
+            g_free(name2);
+            }
+            break;
         default:
-        g_return_val_if_reached(0);
+            g_return_val_if_reached(0);
     }
     return ret;
 }
@@ -197,39 +197,51 @@ static gint find_summary_data(error_equiv_table *err, const expert_info_t *exper
         return -1;
     }
     for (i=0;i<err->num_procs;i++) {
-        if (strcmp(err->procedures[i].entries[2], expert_data->summary) == 0) {
+        if (strcmp(err->procedures[i].entries[1], expert_data->protocol) == 0 &&
+            strcmp(err->procedures[i].entries[2], expert_data->summary) == 0) {
             return i;
         }
     }
     return -1;
 }
 
-/* action is encoded as 
-   filter_action*256+filter_type
+/* Filter actions */
+#define ACTION_MATCH		0
+#define ACTION_PREPARE		1
+#define ACTION_FIND_FRAME	2
+#define ACTION_FIND_NEXT	3
+#define ACTION_FIND_PREVIOUS	4
+#define ACTION_COLORIZE		5
+#define ACTION_WEB_LOOKUP	6
+
+/* Action type - says what to do with the filter */
+#define	ACTYPE_SELECTED		0
+#define ACTYPE_NOT_SELECTED	1
+#define ACTYPE_AND_SELECTED	2
+#define ACTYPE_OR_SELECTED	3
+#define ACTYPE_AND_NOT_SELECTED	4
+#define ACTYPE_OR_NOT_SELECTED	5
+
+/* Encoded callback arguments */
+#define CALLBACK_MATCH(type)		((ACTION_MATCH<<8) | (type))
+#define CALLBACK_PREPARE(type)		((ACTION_PREPARE<<8) | (type))
+#define CALLBACK_FIND_FRAME(type)	((ACTION_FIND_FRAME<<8) | (type))
+#define CALLBACK_FIND_NEXT(type)	((ACTION_FIND_NEXT<<8) | (type))
+#define CALLBACK_FIND_PREVIOUS(type)	((ACTION_FIND_PREVIOUS<<8) | (type))
+#define CALLBACK_COLORIZE(type)		((ACTION_COLORIZE<<8) | (type))
+#define CALLBACK_WEB_LOOKUP		(ACTION_WEB_LOOKUP<<8)
+
+/* Extract components of callback argument */
+#define FILTER_ACTION(cb_arg)		(((cb_arg)>>8) & 0xff)
+#define FILTER_ACTYPE(cb_arg)		((cb_arg) & 0xff)
 
-   filter_action:
-	0: Match
-	1: Prepare
-	2: Find Frame
-	3:   Find Next
-	4:   Find Previous
-	5: Colorize Procedure
-    6: Lookup on Internet
-   filter_type:
-	0: Selected
-	1: Not Selected
-	2: And Selected
-	3: Or Selected
-	4: And Not Selected
-	5: Or Not Selected
-*/
 static void
 error_select_filter_cb(GtkWidget *widget _U_, gpointer callback_data, guint callback_action)
 {
-	int action, type, selection;
-	error_equiv_table *err = (error_equiv_table *)callback_data;
-	char str[256];
-	const char *current_filter;
+    int action, type, selection;
+    error_equiv_table *err = (error_equiv_table *)callback_data;
+    char str[256];
+    const char *current_filter;
 
 #if (GTK_MAJOR_VERSION >= 2)
     GtkTreeIter iter;
@@ -237,12 +249,12 @@ error_select_filter_cb(GtkWidget *widget _U_, gpointer callback_data, guint call
     const expert_info_t expert_data;
 #endif
 
-    action=(callback_action>>8)&0xff;
-	type=callback_action&0xff;
+    action=FILTER_ACTION(callback_action);
+    type=FILTER_ACTYPE(callback_action);
 
 
 #if (GTK_MAJOR_VERSION < 2)
-   	selection=GPOINTER_TO_INT(g_list_nth_data(GTK_CLIST(err->table)->selection, 0));
+    selection=GPOINTER_TO_INT(g_list_nth_data(GTK_CLIST(err->table)->selection, 0));
 #else
     gtk_tree_selection_get_selected(err->select, &model, &iter);
 
@@ -251,23 +263,23 @@ error_select_filter_cb(GtkWidget *widget _U_, gpointer callback_data, guint call
     gtk_tree_model_get (model, &iter, SUMMARY_COLUMN, &expert_data.summary, -1);
     
     if (strcmp((char *)(unsigned long)expert_data.group, "Packet:")==0) {
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "You cannot filter or search for packet number. Click on a valid item header.");
+        simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "You cannot filter or search for packet number. Click on a valid item header.");
         return;
     }
 
     selection = find_summary_data(err, &expert_data);
 #endif
 
-	if(selection>=(int)err->num_procs){
-		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "No items are selected");
-		return;
-	}
+    if(selection>=(int)err->num_procs){
+        simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "No items are selected");
+        return;
+    }
 #if (GTK_MAJOR_VERSION < 2)
-	/* translate it back from row index to index in procedures array */
+    /* translate it back from row index to index in procedures array */
     selection=GPOINTER_TO_INT(gtk_clist_get_row_data(err->table, selection));
 #endif
 
-	current_filter=gtk_entry_get_text(GTK_ENTRY(main_display_filter_widget));
+    current_filter=gtk_entry_get_text(GTK_ENTRY(main_display_filter_widget));
 
     /* Some expert data doesn't pass an expert item. Without this we cannot create a filter */
     /* But allow for searching of internet for error string */
@@ -278,9 +290,8 @@ error_select_filter_cb(GtkWidget *widget _U_, gpointer callback_data, guint call
                 return;
             }
         }
-    	switch(type){
-    	case 0:
-    		/* selected */
+        switch(type){
+        case ACTYPE_SELECTED:
             /* if no expert item was passed */
             if (err->procedures[selection].fvalue_value==NULL) {
                 g_snprintf(str, 255, "%s", err->procedures[selection].entries[2]);
@@ -290,9 +301,8 @@ error_select_filter_cb(GtkWidget *widget _U_, gpointer callback_data, guint call
                 /* expert item exists. Use it. */
                 g_snprintf(str, 255, "%s", err->procedures[selection].fvalue_value);
             }
-    		break;
-    	case 1:
-    		/* not selected */
+            break;
+        case ACTYPE_NOT_SELECTED:
             /* if no expert item was passed */
             if (err->procedures[selection].fvalue_value==NULL) {
                 g_snprintf(str, 255, "!%s", err->procedures[selection].entries[2]);
@@ -302,50 +312,42 @@ error_select_filter_cb(GtkWidget *widget _U_, gpointer callback_data, guint call
                 /* expert item exists. Use it. */
                 g_snprintf(str, 255, "!(%s)", err->procedures[selection].fvalue_value);
             }
-    		break;
+            break;
             /* the remaining cases will only exist if the expert item exists so no need to check */
-    	case 2:
-    		/* and selected */
-    		g_snprintf(str, 255, "(%s) && (%s)", current_filter, err->procedures[selection].fvalue_value);
-    		break;
-    	case 3:
-    		/* or selected */
-    		g_snprintf(str, 255, "(%s) || (%s)", current_filter, err->procedures[selection].fvalue_value);
-    		break;
-    	case 4:
-    		/* and not selected */
-    		g_snprintf(str, 255, "(%s) && !(%s)", current_filter, err->procedures[selection].fvalue_value);
-    		break;
-    	case 5:
-    		/* or not selected */
-    		g_snprintf(str, 255, "(%s) || !(%s)", current_filter, err->procedures[selection].fvalue_value);
-    		break;
+        case ACTYPE_AND_SELECTED:
+            g_snprintf(str, 255, "(%s) && (%s)", current_filter, err->procedures[selection].fvalue_value);
+            break;
+        case ACTYPE_OR_SELECTED:
+            g_snprintf(str, 255, "(%s) || (%s)", current_filter, err->procedures[selection].fvalue_value);
+            break;
+        case ACTYPE_AND_NOT_SELECTED:
+            g_snprintf(str, 255, "(%s) && !(%s)", current_filter, err->procedures[selection].fvalue_value);
+            break;
+        case ACTYPE_OR_NOT_SELECTED:
+            g_snprintf(str, 255, "(%s) || !(%s)", current_filter, err->procedures[selection].fvalue_value);
+            break;
         default:
             simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Can't find menu type - %u", type);
-    	}
+        }
     }
 
-	switch(action){
-	case 0:
-		/* match */
-		main_filter_packets(&cfile, str, FALSE);
+    switch(action){
+    case ACTION_MATCH:
+        main_filter_packets(&cfile, str, FALSE);
         break;
-	case 1:
-		/* prepare */
+    case ACTION_PREPARE:
         gtk_entry_set_text(GTK_ENTRY(main_display_filter_widget), str);
-		break;
-	case 2:
-		/* find frame */
+        break;
+    case ACTION_FIND_FRAME:
         /* When trying to perform a find without expert item, we must pass
          * the expert string to the find window. The user might need to modify
          * the string and click on the text search to locate the packet in question.
          * So regardless of the type we will just bring up the find window and allow
          * the user to modify the search criteria and options.
          */
-            find_frame_with_filter(str);
-		break;
-	case 3:
-		/* find next */
+        find_frame_with_filter(str);
+        break;
+    case ACTION_FIND_NEXT:
         /* In the case of find next, if there was no expert item, then most likely the expert
          * string was modified to locate the text inside the message. So we can't just perform
          * a find with the expert string or we will not really be performing a find next.
@@ -360,11 +362,10 @@ error_select_filter_cb(GtkWidget *widget _U_, gpointer callback_data, guint call
         else
         { 
             /* We have an expert item so just continue search without find dialog. */
-		    find_previous_next_frame_with_filter(str, FALSE);
+            find_previous_next_frame_with_filter(str, FALSE);
         }
-		break;
-	case 4:
-		/* find previous */
+        break;
+    case ACTION_FIND_PREVIOUS:
         /* In the case of find previous, if there was no expert item, then most likely the expert
          * string was modified to locate the text inside the message. So we can't just perform
          * a find with the expert string or we will not really be performing a find previous.
@@ -379,18 +380,17 @@ error_select_filter_cb(GtkWidget *widget _U_, gpointer callback_data, guint call
         else
         { 
             /* We have an expert item so just continue search without find dialog. */
-		    find_previous_next_frame_with_filter(str, TRUE);
+            find_previous_next_frame_with_filter(str, TRUE);
         }
-		break;
-	case 5:
-		/* colorize procedure */
-		color_display_with_filter(str);
-		break;
-	case 6:
-		/* Lookup expert string on internet. Default search via www.google.com */
-		g_snprintf(str, 255, "http://www.google.com/search?hl=en&q=%s+'%s'", err->procedures[selection].entries[1], err->procedures[selection].entries[2]);
+        break;
+    case ACTION_COLORIZE:
+        color_display_with_filter(str);
+        break;
+    case ACTION_WEB_LOOKUP:
+        /* Lookup expert string on internet. Default search via www.google.com */
+        g_snprintf(str, 255, "http://www.google.com/search?hl=en&q=%s+'%s'", err->procedures[selection].entries[1], err->procedures[selection].entries[2]);
         browser_open_url(str);
-		break;
+        break;
 #if (GTK_MAJOR_VERSION < 2)
     case 7:
         /* Goto the first occurance (packet) in the trace */
@@ -399,8 +399,7 @@ error_select_filter_cb(GtkWidget *widget _U_, gpointer callback_data, guint call
 #endif
     default:
         simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Can't find menu action - %u", action);
-	}
-
+    }
 }
 
 static gint
@@ -421,64 +420,84 @@ static GtkItemFactoryEntry error_list_menu_items[] =
 	/* Match */
 	ITEM_FACTORY_ENTRY("/Apply as Filter", NULL, NULL, 0, "<Branch>", NULL),
 	ITEM_FACTORY_ENTRY("/Apply as Filter/Selected", NULL,
-		error_select_filter_cb, 0*256+0, NULL, NULL),
+		error_select_filter_cb, CALLBACK_MATCH(ACTYPE_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Apply as Filter/... not Selected", NULL,
-		error_select_filter_cb, 0*256+1, NULL, NULL),
+		error_select_filter_cb, CALLBACK_MATCH(ACTYPE_NOT_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Apply as Filter/.. and Selected", NULL,
-		error_select_filter_cb, 0*256+2, NULL, NULL),
+		error_select_filter_cb, CALLBACK_MATCH(ACTYPE_AND_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Apply as Filter/... or Selected", NULL,
-		error_select_filter_cb, 0*256+3, NULL, NULL),
+		error_select_filter_cb, CALLBACK_MATCH(ACTYPE_OR_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Apply as Filter/... and not Selected", NULL,
-		error_select_filter_cb, 0*256+4, NULL, NULL),
+		error_select_filter_cb, CALLBACK_MATCH(ACTYPE_AND_NOT_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Apply as Filter/... or not Selected", NULL,
-		error_select_filter_cb, 0*256+5, NULL, NULL),
+		error_select_filter_cb, CALLBACK_MATCH(ACTYPE_OR_NOT_SELECTED),
+		NULL, NULL),
 
 	/* Prepare */
 	ITEM_FACTORY_ENTRY("/Prepare a Filter", NULL, NULL, 0, "<Branch>", NULL),
 	ITEM_FACTORY_ENTRY("/Prepare a Filter/Selected", NULL,
-		error_select_filter_cb, 1*256+0, NULL, NULL),
+		error_select_filter_cb, CALLBACK_PREPARE(ACTYPE_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Prepare a Filter/Not Selected", NULL,
-		error_select_filter_cb, 1*256+1, NULL, NULL),
+		error_select_filter_cb, CALLBACK_PREPARE(ACTYPE_NOT_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Prepare a Filter/... and Selected", NULL,
-		error_select_filter_cb, 1*256+2, NULL, NULL),
+		error_select_filter_cb, CALLBACK_PREPARE(ACTYPE_AND_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Prepare a Filter/... or Selected", NULL,
-		error_select_filter_cb, 1*256+3, NULL, NULL),
+		error_select_filter_cb, CALLBACK_PREPARE(ACTYPE_OR_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Prepare a Filter/... and not Selected", NULL,
-		error_select_filter_cb, 1*256+4, NULL, NULL),
+		error_select_filter_cb, CALLBACK_PREPARE(ACTYPE_AND_NOT_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Prepare a Filter/... or not Selected", NULL,
-		error_select_filter_cb, 1*256+5, NULL, NULL),
+		error_select_filter_cb, CALLBACK_PREPARE(ACTYPE_OR_NOT_SELECTED),
+		NULL, NULL),
 
 	/* Find Frame */
 	ITEM_FACTORY_ENTRY("/Find Frame", NULL, NULL, 0, "<Branch>", NULL),
 	ITEM_FACTORY_ENTRY("/Find Frame/Find Frame", NULL, NULL, 0, "<Branch>", NULL),
 	ITEM_FACTORY_ENTRY("/Find Frame/Find Frame/Selected", NULL,
-		error_select_filter_cb, 2*256+0, NULL, NULL),
+		error_select_filter_cb, CALLBACK_FIND_FRAME(ACTYPE_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Find Frame/Find Frame/Not Selected", NULL,
-		error_select_filter_cb, 2*256+1, NULL, NULL),
+		error_select_filter_cb, CALLBACK_FIND_FRAME(ACTYPE_NOT_SELECTED),
+		NULL, NULL),
 	/* Find Next */
 	ITEM_FACTORY_ENTRY("/Find Frame/Find Next", NULL, NULL, 0, "<Branch>", NULL),
 	ITEM_FACTORY_ENTRY("/Find Frame/Find Next/Selected", NULL,
-		error_select_filter_cb, 3*256+0, NULL, NULL),
+		error_select_filter_cb, CALLBACK_FIND_NEXT(ACTYPE_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Find Frame/Find Next/Not Selected", NULL,
-		error_select_filter_cb, 3*256+1, NULL, NULL),
+		error_select_filter_cb, CALLBACK_FIND_NEXT(ACTYPE_NOT_SELECTED),
+		NULL, NULL),
 
 	/* Find Previous */
 	ITEM_FACTORY_ENTRY("/Find Frame/Find Previous", NULL, NULL, 0, "<Branch>", NULL),
 	ITEM_FACTORY_ENTRY("/Find Frame/Find Previous/Selected", NULL,
-		error_select_filter_cb, 4*256+0, NULL, NULL),
+		error_select_filter_cb, CALLBACK_FIND_PREVIOUS(ACTYPE_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Find Frame/Find Previous/Not Selected", NULL,
-		error_select_filter_cb, 4*256+1, NULL, NULL),
+		error_select_filter_cb, CALLBACK_FIND_PREVIOUS(ACTYPE_NOT_SELECTED),
+		NULL, NULL),
 
 	/* Colorize Procedure */
 	ITEM_FACTORY_ENTRY("/Colorize Procedure", NULL, NULL, 0, "<Branch>", NULL),
 	ITEM_FACTORY_ENTRY("/Colorize Procedure/Selected", NULL,
-		error_select_filter_cb, 5*256+0, NULL, NULL),
+		error_select_filter_cb, CALLBACK_COLORIZE(ACTYPE_SELECTED),
+		NULL, NULL),
 	ITEM_FACTORY_ENTRY("/Colorize Procedure/Not Selected", NULL,
-		error_select_filter_cb, 5*256+1, NULL, NULL),
+		error_select_filter_cb, CALLBACK_COLORIZE(ACTYPE_NOT_SELECTED),
+		NULL, NULL),
 
 	/* Search Internet */
 	ITEM_FACTORY_ENTRY("/Internet Search for Info Text", NULL,
-		error_select_filter_cb, 6*256+0, NULL, NULL),
+		error_select_filter_cb, CALLBACK_WEB_LOOKUP, NULL, NULL),
 #if (GTK_MAJOR_VERSION < 2)
    	/* Go to first packet matching this entry */
 	ITEM_FACTORY_ENTRY("/Goto First Occurrence", NULL,
@@ -512,7 +531,7 @@ expert_goto_pkt_cb (GtkTreeSelection *selection, gpointer data _U_)
 static void
 error_create_popup_menu(error_equiv_table *err)
 {
-	GtkItemFactory *item_factory;
+    GtkItemFactory *item_factory;
 
 
 #if (GTK_MAJOR_VERSION >= 2)
@@ -522,15 +541,15 @@ error_create_popup_menu(error_equiv_table *err)
                   G_CALLBACK (expert_goto_pkt_cb),
                   err);
 #endif
-	item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<main>", NULL);
+    item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<main>", NULL);
 
-	gtk_item_factory_create_items_ac(item_factory, sizeof(error_list_menu_items)/sizeof(error_list_menu_items[0]), error_list_menu_items, err, 2);
+    gtk_item_factory_create_items_ac(item_factory, sizeof(error_list_menu_items)/sizeof(error_list_menu_items[0]), error_list_menu_items, err, 2);
 
-	err->menu = gtk_item_factory_get_widget(item_factory, "<main>");
+    err->menu = gtk_item_factory_get_widget(item_factory, "<main>");
 #if (GTK_MAJOR_VERSION >= 2)
-	SIGNAL_CONNECT(err->tree_view, "button_press_event", error_show_popup_menu_cb, err);
+    SIGNAL_CONNECT(err->tree_view, "button_press_event", error_show_popup_menu_cb, err);
 #else
-	SIGNAL_CONNECT(err->table, "button_press_event", error_show_popup_menu_cb, err);
+    SIGNAL_CONNECT(err->table, "button_press_event", error_show_popup_menu_cb, err);
 #endif
 }
 
@@ -774,7 +793,7 @@ init_error_table_row(error_equiv_table *err, const expert_info_t *expert_data)
 void
 add_error_table_data(error_equiv_table *err, const expert_info_t *expert_data)
 {
-	error_procedure_t *errp;
+    error_procedure_t *errp;
     gint index;
 #if (GTK_MAJOR_VERSION < 2)
     gint row;
@@ -792,13 +811,13 @@ add_error_table_data(error_equiv_table *err, const expert_info_t *expert_data)
         simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, "Could not find expert data. Aborting");
         return;
     }
-	errp=&err->procedures[index];
+    errp=&err->procedures[index];
 
 #if (GTK_MAJOR_VERSION < 2)
-   	if (errp->count==0){
-		row=gtk_clist_append(err->table, err->procedures[index].entries);
-		gtk_clist_set_row_data(err->table, row, GINT_TO_POINTER(index));
-	}
+    if (errp->count==0){
+        row=gtk_clist_append(err->table, err->procedures[index].entries);
+        gtk_clist_set_row_data(err->table, row, GINT_TO_POINTER(index));
+    }
     errp->count++;
     err->procedures[index].entries[3] = (char *)g_strdup_printf("%d", errp->count);
 #else
@@ -842,26 +861,26 @@ draw_error_table_data(error_equiv_table *err)
 void
 reset_error_table_data(error_equiv_table *err)
 {
-	guint16 i;
+    guint16 i;
 #if (GTK_MAJOR_VERSION >= 2)
     GtkTreeStore    *store;
 #endif
 
-	for(i=0;i<err->num_procs;i++){
-		err->procedures[i].entries[0] = NULL;
-		err->procedures[i].entries[1] = NULL;
-		err->procedures[i].entries[2] = NULL;
-		err->procedures[i].entries[3] = NULL;
+    for(i=0;i<err->num_procs;i++){
+        err->procedures[i].entries[0] = NULL;
+        err->procedures[i].entries[1] = NULL;
+        err->procedures[i].entries[2] = NULL;
+        err->procedures[i].entries[3] = NULL;
 #if (GTK_MAJOR_VERSION < 2)
         err->procedures[i].packet_num=0;
 #else
         err->procedures[i].count=0;
 #endif
 
-	}
+    }
 
 #if (GTK_MAJOR_VERSION < 2)
-	gtk_clist_clear(err->table);
+    gtk_clist_clear(err->table);
 #else
     store = GTK_TREE_STORE(gtk_tree_view_get_model(err->tree_view));
     gtk_tree_store_clear(store);
@@ -872,13 +891,13 @@ reset_error_table_data(error_equiv_table *err)
 void
 free_error_table_data(error_equiv_table *err)
 {
-	guint16 i,j;
+    guint16 i,j;
 
-	for(i=0;i<err->num_procs;i++){
-		for(j=0;j<4;j++){
-			if(err->procedures[i].entries[j]){
-				err->procedures[i].entries[j]=NULL;
-			}
+    for(i=0;i<err->num_procs;i++){
+        for(j=0;j<4;j++){
+            if(err->procedures[i].entries[j]){
+                err->procedures[i].entries[j]=NULL;
+            }
             err->procedures[i].fvalue_value=NULL;
 
 #if (GTK_MAJOR_VERSION < 2)
@@ -886,8 +905,8 @@ free_error_table_data(error_equiv_table *err)
 #else
             err->procedures[i].count=0;
 #endif
-		}
-	}
-	err->procedures=NULL;
-	err->num_procs=0;
+        }
+    }
+    err->procedures=NULL;
+    err->num_procs=0;
 }
