@@ -1,6 +1,6 @@
 /* libpcap.c
  *
- * $Id: libpcap.c 24622 2008-03-14 00:01:51Z gerald $
+ * $Id: libpcap.c 27265 2009-01-18 21:07:06Z gerald $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -461,6 +461,14 @@ static const struct {
 	{ 211, 		WTAP_ENCAP_MOST },
 	/* LIN frame */
 	{ 212, 		WTAP_ENCAP_LIN },
+	/* X2E Xoraya serial frame */
+	{ 213, 		WTAP_ENCAP_X2E_SERIAL },
+	/* X2E Xoraya frame */
+	{ 214, 		WTAP_ENCAP_X2E_XORAYA },
+	/* IEEE 802.15.4 Wireless PAN non-ASK PHY */
+	{ 215,		WTAP_ENCAP_IEEE802_15_4_NONASK_PHY },
+	/* USB packets with padded Linux-specified header */
+	{ 220, 		WTAP_ENCAP_USB_LINUX_MMAPPED },
 
 	/*
 	 * To repeat:
@@ -1423,6 +1431,7 @@ static gboolean libpcap_read(wtap *wth, int *err, gchar **err_info,
 		break;
 
 	case WTAP_ENCAP_USB_LINUX:
+	case WTAP_ENCAP_USB_LINUX_MMAPPED:
 		if (packet_size < sizeof (struct linux_usb_phdr)) {
 			/*
 			 * Uh-oh, the packet isn't big enough to even
@@ -1636,6 +1645,7 @@ libpcap_seek_read(wtap *wth, gint64 seek_off,
 		break;
 
 	case WTAP_ENCAP_USB_LINUX:
+	case WTAP_ENCAP_USB_LINUX_MMAPPED:
 		if (!libpcap_read_linux_usb_pseudoheader(wth, wth->random_fh,
 		    pseudo_header, err))
 			return FALSE;	/* Read error */
@@ -2480,6 +2490,7 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		break;
 
 	case WTAP_ENCAP_USB_LINUX:
+	case WTAP_ENCAP_USB_LINUX_MMAPPED:
 		hdrsize = sizeof (struct linux_usb_phdr);
 		break;
 
@@ -2718,6 +2729,7 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		break;
 
 	case WTAP_ENCAP_USB_LINUX:
+	case WTAP_ENCAP_USB_LINUX_MMAPPED:
 		/*
 		 * Write out the pseudo-header; it has the same format
 		 * as the Linux USB header, and that header is supposed
